import pygame
import os
import random
from random import shuffle

screen_width = 750
screen_height = 750
game_window = pygame.display.set_mode((screen_width, screen_height))
pygame.display.set_caption("Grand Theft Stappenplan")  # Set caption
pygame.font.init()  # Initialise font

# Background main loop
background = pygame.transform.scale(pygame.image.load(os.path.join("assets", "street.jpg")),
                                    (screen_width, screen_height))
# Menu background
menu_background = pygame.transform.scale(pygame.image.load(os.path.join("assets", "menu_bg.jpg")),
                                         (screen_width, screen_height))

main = False  # boolean that becomes true when the player enter the main loop
menu = True  # menu boolean that becomes false when the player leaves the menu
frames_per_second = 60  # number of frames per second
start_time_in_seconds = 0  # the time in seconds the player starts with the main loop
answer_time = 15  # the time in seconds the user gets to answer the questions
obstacle_time = 10  # the time in seconds in between the questions
bQuestion = False  # boolean that becomes true while a question is being asked
bGameOver = False  # boolean that becomes true when the player gets out of the main loop
bInstruction = False  # boolean that becomes true when the player is in the instruction menu
bInputName = False  # boolean that becomes true when the player is in the name input menu
user_text = ""
question_number = 0  # index to keep track how many questions the player has answered

scores = []  # list that will contain all the scores
question_list = []  # list that will contain all the question objects
obstacles = []  # list that will contain all the obstacle objects
background_y = 0


class Player:  # player class with x position, y position, velocity and name
    def __init__(self, x, y, vel, name):
        self.x = x
        self.y = y
        self.car_img = pygame.image.load(os.path.join("assets", "car.png"))
        self.car_img = pygame.transform.scale(self.car_img, (250, 250))
        self.mask = pygame.mask.from_surface(self.car_img)
        self.name = name
        self.velocity = vel
        self.score = 0

    def draw(self, window):
        window.blit(self.car_img, (self.x, self.y))

    def get_width(self):
        return self.car_img.get_width()

    def get_height(self):
        return self.car_img.get_height()


class Obstacle:  # obstacle class with x position, y position and velocity
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.obstacle_img = pygame.image.load(os.path.join("assets", "barrel.png"))
        self.mask = pygame.mask.from_surface(self.obstacle_img)
        self.velocity = 5

    def draw(self, window):
        window.blit(self.obstacle_img, (self.x, self.y))

    def move(self, vel):
        self.y += vel

    def get_height(self):
        return self.obstacle_img.get_height()


class Question:  # question class with question prompt, answers a-d and the correct answer
    def __init__(self, prompt, a, b, c, d, correct_answer):
        self.prompt = prompt
        self.a = a
        self.b = b
        self.c = c
        self.d = d
        self.correct_answer = correct_answer


# Loading all questions into a list
f = open('gts_questions.txt', 'r')

for line in f:
    line = line.split('&')
    line[5] = line[5].replace('\n', '')
    question = Question(line[0], line[1], line[2], line[3], line[4], int(line[5]))
    question_list.append(question)

f.close()

shuffle(question_list)  # Randomize question order


# Fonts
main_font = pygame.font.SysFont("comicsans", 50)
question_font = pygame.font.SysFont("comicsans", 35)
menu_font = pygame.font.SysFont("forte", 90)
game_over_font = pygame.font.SysFont("forte", 50)


clock = pygame.time.Clock()


# function that handles drawing of player, obstacles and text labels in the main loop
def redraw_window():

    if not bQuestion:
        for obstacle in obstacles:
            obstacle.draw(game_window)
    else:
        display_text(main_font, question_list[question_number].prompt, 10, 50)
        display_text(main_font, "A", 50, 150)
        display_text(main_font, "B", 250, 150)
        display_text(main_font, "C", 450, 150)
        display_text(main_font, "D", 650, 150)
        display_text(question_font, question_list[question_number].a, 10, 200, 180)
        display_text(question_font, question_list[question_number].b, 180, 200, 370)
        display_text(question_font, question_list[question_number].c, 380, 200, 570)
        display_text(question_font, question_list[question_number].d, 580, 200)

    score_label = main_font.render(f"Score: {player.score}", 1, (255, 255, 255))
    display_text(main_font, "Time: " + str(time_in_seconds), 10, 10)
    display_text(main_font, "Score: " + str(player.score), screen_width - score_label.get_width() - 10, 10)

    player.draw(game_window)

    pygame.display.update()


# function that handles the collision between two objects with masks
def collide(object1, object2):
    offset_x = object2.x - object1.x
    offset_y = object2.y - object1.y
    return object1.mask.overlap(object2.mask, (offset_x, offset_y))


# function that handles drawing text onscreen and cuts it off when the label exceeds the boundary
def display_text(font, text, x, y, boundary=screen_width):
    words = [word.split(' ') for word in text.splitlines()]
    space = font.size(' ')[0]  # The width of a space.
    word_x = x
    word_y = y
    for line in words:
        for word in line:
            word_surface = font.render(word, 0, (255, 255, 255))
            word_width, word_height = word_surface.get_size()
            if word_x + word_width >= boundary:
                word_x = x  # Reset the x.
                word_y += word_height  # Start on new row.
            game_window.blit(word_surface, (word_x, word_y))
            word_x += word_width + space
        word_x = x  # Reset the x.
        word_y += word_height  # Start on new row.


# quits the application when the user presses the escape key or clicks on the quit button
def quit_check():
    keys = pygame.key.get_pressed()
    for event in pygame.event.get():
        if event.type == pygame.QUIT or keys[pygame.K_ESCAPE]:
            pygame.quit()


# menu loop that runs while the user is in the menu
while menu:
    game_window.blit(menu_background, (0, 0))
    start_label = menu_font.render("Play", 1, (255, 255, 255))
    instructions_label = menu_font.render("Instructions", 1, (255, 255, 255))
    back_label = menu_font.render("Back", 1, (255, 255, 255))
    input_box = pygame.Rect(100, 300, 550, 100)

    if not bInstruction and not bInputName:
        display_text(menu_font, "Play", int(screen_width / 2 - .5 * start_label.get_width()), 100)
        display_text(menu_font, "Instructions", 130, 300)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                mouse = pygame.mouse.get_pos()

                if mouse[0] in range(int(screen_width / 2 - .5 * start_label.get_width()),
                                     int(screen_width / 2 - .5 * start_label.get_width()) + start_label.get_width()) \
                        and mouse[1] in range(100, 100 + start_label.get_height()):
                    bInputName = True
                elif mouse[0] in range(130, 130 + instructions_label.get_width()) \
                        and mouse[1] in range(300, 300 + instructions_label.get_height()):
                    bInstruction = True

    elif bInstruction:
        display_text(game_over_font, "Use the arrow keys to move and try to avoid the obstacles.", 35, 100)
        display_text(game_over_font, "When a question appears drive to the road with the right answer.", 35, 300)
        display_text(menu_font, "Back", int(screen_width / 2 - .5 * back_label.get_width()), 500)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                mouse = pygame.mouse.get_pos()

                if mouse[0] in range(int(screen_width / 2 - .5 * back_label.get_width()),
                                     int(screen_width / 2 - .5 * back_label.get_width()) + back_label.get_width()) \
                        and mouse[1] in range(500, 500 + back_label.get_height()):
                    bInstruction = False
    elif bInputName:
        pygame.draw.rect(game_window, (255, 255, 255), input_box, 2)
        display_text(menu_font, "Back", int(screen_width / 2 - .5 * back_label.get_width()), 500)
        display_text(game_over_font, "Type your name and press enter to play.", 100, 100)
        display_text(menu_font, user_text, input_box.x, input_box.y)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_BACKSPACE:
                    user_text = user_text[: -1]
                elif event.key == pygame.K_RETURN and user_text != "":
                    player = Player(300, 300, 5, user_text)
                    start_time_in_seconds = pygame.time.get_ticks() // 1000
                    menu = False
                    main = True
                elif len(user_text) <= 11:
                    user_text += event.unicode
            if event.type == pygame.MOUSEBUTTONDOWN:
                mouse = pygame.mouse.get_pos()

                if mouse[0] in range(int(screen_width / 2 - .5 * back_label.get_width()),
                                     int(screen_width / 2 - .5 * back_label.get_width()) + back_label.get_width()) \
                        and mouse[1] in range(500, 500 + back_label.get_height()):
                    bInputName = False
    pygame.display.update()


while main:
    clock.tick(frames_per_second)
    # time in seconds subtracted by the time when the main loop starts
    time_in_seconds = pygame.time.get_ticks() // 1000 - start_time_in_seconds

    # player movement with the arrow keys
    keys = pygame.key.get_pressed()
    if keys[pygame.K_LEFT] and player.x - player.velocity + 70 > 0:
        player.x -= player.velocity
    if keys[pygame.K_RIGHT] and player.x + player.velocity + player.get_width() - 70 < screen_width:
        player.x += player.velocity
    if keys[pygame.K_DOWN] and player.y + player.velocity + player.get_height() < screen_height:
        player.y += player.velocity
    if keys[pygame.K_UP] and player.y - player.velocity > 0:
        player.y -= player.velocity

    # only handle obstacle spawn and collision when there isn't a question being asked
    # only set the start_timer when there isn't a question being asked
    if not bQuestion:
        # when obstacle time has passed bQuestion becomes true
        if time_in_seconds % obstacle_time == 0 and time_in_seconds != 0:
            start_timer = time_in_seconds
            bQuestion = True

        # spawn new obstacles when the obstacle list is empty
        if len(obstacles) == 0:
            for i in range(5):
                obstacle = Obstacle(random.randrange(50, screen_width - 100), random.randrange(-1500, -100))
                obstacles.append(obstacle)

        for obstacle in obstacles[:]:
            obstacle.move(obstacle.velocity)

            if collide(obstacle, player):
                player.score -= 10
                obstacles.remove(obstacle)
            elif obstacle.y + obstacle.get_height() > screen_height:
                obstacles.remove(obstacle)
    else:
        # when a question is being asked clear the obstacles
        # and check the player x position after the answer time has expired
        obstacles.clear()
        if time_in_seconds >= start_timer + answer_time:
            if question_list[question_number].correct_answer == 0 and player.x + player.get_width() / 2 <= 168:
                player.score += 200
            elif question_list[question_number].correct_answer == 1 \
                    and 368 >= player.x + player.get_width() / 2 >= 168:
                player.score += 200
            elif question_list[question_number].correct_answer == 2 \
                    and 564 >= (player.x + player.get_width() / 2) >= 368:
                player.score += 200
            elif question_list[question_number].correct_answer == 3 and player.x + player.get_width() / 2 >= 564:
                player.score += 200

            question_number += 1
            bQuestion = False
            # check if all the questions in the list were asked
            if question_number >= len(question_list):
                bGameOver = True
                main = False

                # Save score
                with open('highscores.txt', 'a') as f:
                    f.write(player.name + '&' + str(player.score) + '\n')

                # Show high scores
                f = open('highscores.txt', 'r')

                for line in f:
                    scores.append(line.split('&'))
                    # check if txt file is not empty
                    if len(scores[len(scores) - 1]) > 0:
                        scores[len(scores) - 1][1] = int(scores[len(scores) - 1][1].replace('\n', ''))

                if len(scores) < 3:
                    for i in range(0, 3 - len(scores)):
                        scores.append(["None", 0])

                f.close()

                scores = sorted(scores, key=lambda l: l[1], reverse=True)

    # moves the background down and draws a new background on top of the old one
    relative_background_y = background_y % background.get_rect().height
    game_window.blit(background, (0, relative_background_y - background.get_rect().height))

    if relative_background_y < screen_width:
        game_window.blit(background, (0, relative_background_y))
    background_y = background_y + player.velocity

    redraw_window()

    quit_check()

# display the highscore and the player score
while bGameOver:
    game_window.blit(menu_background, (0, 0))
    display_text(game_over_font, "Highscores:", 100, 200)

    display_text(game_over_font, ("1 " + str(scores[0][0]) + ' with score ' + str(scores[0][1])), 100, 300)
    display_text(game_over_font, ("2 " + str(scores[1][0]) + ' with score ' + str(scores[1][1])), 100, 400)
    display_text(game_over_font, ("3 " + str(scores[2][0]) + ' with score ' + str(scores[2][1])), 100, 500)
    display_text(game_over_font, "Your score was " + str(player.score), 100, 100)
    pygame.display.update()

    quit_check()
